#! /usr/bin/python
# -*- coding: utf-8 -*-
from foqus.customers import *
from foqus.pipline_cutomise import Pipeline2
from foqus.request_api import APIFoqus
from six.moves import http_client

from urllib.parse import urlparse
from jsondiff import diff

import imghdr
import mimetypes
import requests
import shutil

import os
import json
import glob

import time
import sys

api = APIFoqus()


def get_category(last_json, num_cat):
    for json_value in last_json.values():
        for index_product in range(len(json_value)):
            if str(index_product) == str(num_cat):
                try:
                    logger.info("The_Category_that_will_have_changes: " + str(json_value[index_product]['Categorie']))
                    return str(json_value[index_product]['Categorie'])
                except Exception as e:
                    return None


def add_category_and_write_to_file(status, last_json_path, last_json, new_category_objects, path_new_file):
    try:
        new_obj = dict()
        for k, v in last_json.items():
            for o in range(len(v)):
                for p in range(len(v[o]['Photos'])):
                    photo = v[o]['Photos'][p]
                    photo["Status"] = 2

                for obj in new_category_objects:
                    if v[o] == obj:
                        v[o] = obj
                        photo['Status'] = status
        new_obj[k] = v
        with open(path_new_file, 'w') as outfile:
            json.dump(new_obj, outfile)
        outfile.close()
        logger.info("----delta: added category and wrote to file")
    except Exception as e:
        logger.error("Error in_diff_files add_category_and_write_to_file %s" %e)


def write_to_file(status, last_json, path_new_file, category_will_have_changes=None, block_image_n=None,
                  new_file=False, deleted_block=None):
    if os.path.isfile(path_new_file):
        logger.info("-----path_new_file: %s" % path_new_file)
        with open(str(path_new_file), 'r') as content_file:
            content = content_file.read()
        json_value = json.loads(content)
    else:
        json_value = []
    logger.info("----delta: creating file...")
    new_obj = {}
    key = ""
    try:
        for key, json_value in last_json.items():
            for index_product in range(len(json_value)):
                for product in range(len(json_value[index_product]['Photos'])):
                    if new_file:
                        photo = json_value[index_product]['Photos'][product]
                        photo["Status"] = 1
                    if not block_image_n and not "Status" in json_value[index_product]['Photos'][product] \
                            and category_will_have_changes:
                        photo = json_value[index_product]['Photos'][product]
                        photo["Status"] = 2
                    if category_will_have_changes \
                            and json_value[index_product]['Categorie'] == category_will_have_changes and \
                                    str(product) == str(block_image_n):
                            photo = json_value[index_product]['Photos'][product]
                            photo["Status"] = status
                    if block_image_n:
                        photo = json_value[index_product]['Photos'][int(block_image_n)]
                        photo["Status"] = status
                    if deleted_block and not category_will_have_changes and not "Status" in json_value[index_product]['Photos'][product]:
                        deleted_block['Status'] = status
                        logger.info(deleted_block)
                        json_value[0]['Photos'].append(deleted_block)
                    elif not "Status" in json_value[index_product]['Photos'][product]:
                        photo = json_value[index_product]['Photos'][product]
                        photo["Status"] = status
        new_obj[key] = json_value
        with open(path_new_file, 'w') as outfile:
            json.dump(new_obj, outfile)
        outfile.close()
        logger.info("-----delta file created")
    except Exception as e:
        logger.error("Erooor in_difff_files write_to_file %s" % e)


def delta_retrieve_json(customer_name, customer_type, project_name):
    logger.info("============> delta retrieve json started")
    sys.setrecursionlimit(3000)
    start = time.process_time()
    path_json = STREAMS_PATH + customer_type + '/' + customer_name + '/similars/' + project_name
    date = datetime.datetime.today().strftime("%Y-%m-%d:%H-%M")
    path_new_file = STREAMS_PATH + customer_type + '/' + customer_name + '/delta_files/' + project_name + '/'

    if not os.path.isdir(path_new_file):
        try:
            os.makedirs(path_new_file)
        except Exception as e:
            logger.error("Cannot create delta_files directory for customer. "
                         "Please verify permissions ")
            pass

    new_file_status = path_new_file + date + ".json"

    # if there are shopify files take only shopify_file*.json
    if len(glob.glob(path_json + "/shopify_*.json")) > 0:
        paths = sorted(glob.glob(path_json + "/shopify_file_*.json"), key=os.path.getmtime)
    else:
        paths = sorted(glob.glob(path_json + "/*.json"), key=os.path.getmtime)
    if len(paths) > 1:
        last_json_path = paths[-1]
        before_last_json_path = paths[-2]
        logger.info("******** "+before_last_json_path)
        logger.info("******** "+last_json_path)
        last_json = json.load(open(last_json_path))
        before_last_json = json.load(open(before_last_json_path))
    elif len(paths) == 1:
        last_json_path = paths[-1]
        last_json = json.load(open(last_json_path))
        before_last_json = {customer_name + "_" + customer_type: []}
    else:
        last_json = {customer_name + "_" + customer_type: []}
        before_last_json = {customer_name + "_" + customer_type: []}
    try:
        difference = diff(before_last_json, last_json)
    except Exception as e:
        logger.error("Error_diff %s" %e)
        difference = {}

    if before_last_json[customer_name + "_" + customer_type] != [] and difference != {} \
            and difference[customer_name + '_' + customer_type]:
        logger.info("-----delta: comparing to existing file")
        for key_cat, cat in difference[customer_name + '_' + customer_type].items():
            logger.info("======== key_cat and cat:" + str(key_cat) + str(cat))
            category_will_have_changes = get_category(last_json, key_cat)
            if category_will_have_changes:
                for kphotos, photos in cat.get('Photos', {}).items():
                    if str(kphotos) == '$delete':
                        for deleted_position in photos:
                            block_image_n = str(deleted_position)
                            write_to_file(0, last_json, new_file_status, category_will_have_changes, block_image_n)
                    elif str(kphotos) == '$insert':
                        for position_added, block in photos:
                            status = 1
                            block_image_n = str(position_added)
                            write_to_file(status, last_json, new_file_status, category_will_have_changes,
                                                          block_image_n)
                    else:
                        logger.info("Block already donwloaded")
                        write_to_file(2, last_json, new_file_status)
                        continue
            else:
                try:
                    logger.info("-----delta var = cat:")
                    logger.info(cat)
                    for change_type, change in cat.get('Photos', {}).items():
                        if str(change_type) == '$insert':
                            for position_added, block in change:
                                block_image_n = str(position_added)
                                logger.info("added in: " + str(block_image_n))
                                write_to_file(1, last_json, new_file_status, block_image_n=block_image_n)
                        elif str(change_type) == '$delete':
                            for deleted_position in change:
                                block_image_n = str(deleted_position)
                                deleted_block = before_last_json[customer_name + "_" + customer_type][0]['Photos'][int(block_image_n)]
                                logger.info(deleted_block)
                                write_to_file(0, last_json, new_file_status, block_image_n=block_image_n, deleted_block=deleted_block)
                    write_to_file(2, last_json, new_file_status)
                except AttributeError as err:
                    # Only categories have changed not photos inside them
                    logger.error(err)
                except Exception as e:
                    logger.error("Erooor in_difff_files delta_retrieve_json %s" % e)
                    write_to_file(1, last_json, new_file_status)
    elif difference == {} and len(paths) > 1:
        logger.info("-----delta: same file")
        write_to_file(2, last_json, new_file_status)
    else:
        logger.info("-----delta: 1st time, creating json")
        try:
            write_to_file(1, last_json, new_file_status, new_file=True)
        except Exception as e:
            logger.error(e)
        logger.info("------delta: done creating file")
    logger.info("-----Delta retrieve json compare time taken: "+str(time.process_time() - start))
    return new_file_status


def load_json_data(json_path):
    try:
        with open(json_path) as data_file:
            json_input = json.load(data_file)
    except:
        json_data = open(json_path)
        bom_maybe = json_data.read(3)
        if bom_maybe != codecs.BOM_UTF8:
            json_data.seek(0)
        json_input = json.load(json_data)
    return json_input


def checkUrl(url):
    p = urlparse(url)
    conn = http_client.HTTPConnection(p.netloc)
    conn.request('HEAD', p.path)
    resp = conn.getresponse()
    return resp.status < 400


def is_url_image(image_url):
    image_formats = ("image/png", "image/jpeg", "image/jpg", "binary/octet-stream")
    r = requests.head(image_url)
    if r.headers["content-type"] not in image_formats:
        mimetype, encoding = mimetypes.guess_type(image_url)
        return mimetype and mimetype.startswith('image')
    else:
        return True


def max_nb_images(path):
    list_nbrs = []
    for j in os.listdir(path):
        for k in os.listdir(str(path) + "/" + str(j)):
            ext = imghdr.what(str(path) + "/" + str(j) + "/" + str(k))
            if (ext == 'png' and k.split(".")[1] in ['jpg', 'jpeg', 'JPG', 'JPEG']) or \
                    (ext in ['jpeg', 'jpg'] and k.split('.')[1] in ['png', 'PNG']) or (ext is None):
                os.remove(str(path) + "/" + str(j) + "/" + str(k))
                logger.info("Deleting_image as it contains png encoding - " + str(path) + "/" + str(j) + "/" + str(k))
        list_nbrs.append(len(os.listdir(str(path) + "/" + str(j))))
    return max(list_nbrs)


def verif_folder_less_twenty(path, nb_classes):
    # Not actually verif less than 20 its just the name
    image_ext = ['.jpg', '.png', '.jpeg', '.JPG', '.PNG', '.JPEG']
    folder_less_20 = []
    # we will check if the number returned by max_nb_images is < batch size in this case it will cause error
    # in training so we take max between it and batch/nb_classes
    max_nb = max(round(BATCH_SIZE / nb_classes), max_nb_images(path))
    if max_nb < 20:
        max_nb = 20
    for j in os.listdir(path):
        x = 0
        for i in os.listdir(str(path) + "/" + str(j)):
            if os.path.splitext(i)[1] in image_ext:
                x = x + 1
        if x == 0 :
            shutil.rmtree(str(path) + "/" + str(j))
            logger.info("Deletin Empty folder : " + str(path) + "/" + str(j))
        elif (x <= 20 or x < max_nb):
            logger.info("WARNING: Folder has less images : " + str(j).split('/')[-1])
            folder_less_20.append(str(j).split('/')[-1])
    return folder_less_20, max_nb


def generate_more_images(training_path, folders, max_nb):
    for id_categorie in folders:
        DIR = str(training_path) + "/" + str(id_categorie)
        nb_images = (len(os.listdir(DIR)))
        p = Pipeline2(source_directory=DIR, output_directory=DIR)
        p.rotate(probability=1, max_left_rotation=5, max_right_rotation=5)
        p.flip_left_right(probability=0.5)
        p.zoom_random(probability=0.5, percentage_area=0.8)
        p.flip_top_bottom(probability=0.5)
        p.sample(max_nb - nb_images)
    logger.info("The images increase is done successfully")


def classification_similars(customer_name, customer_type, project_name):
    api.apipost("classification_similars", customer_name, customer_type, project_name)


def process_customer_stream_from_json(customer_name, customer_type, project_name):
    api.apipost('training_similars', customer_name, customer_type, project_name)


def get_redirect_url(url):
    try:
        image = requests.get(url)
        if image.url == url:
            return is_url_image(url)
        else:
            return is_url_image(image.url)
    except:
        return False


def shopify_training(customer_name, customer_type, url_shop, project):
    api.apipost('shopify_training', customer_name, customer_type, None, None, project, url_shop)


def generate_similarity_vector(customer_name="vector", customer_type='vector', deleted_image={}, project_name=None):
    vector_response = api.apipost('training_similars', customer_name, customer_type, project_name, deleted_image, None)
    response_text = json.loads(vector_response.text)
    response_from_parquet = response_text['response']
    if response_from_parquet == 2:
        return True
    return False


def text_training_retrieve_json(excel_path, customer_name, customer_type, customer_universe, project_name):
    send_email_when_training_started(customer_name, project_name, 'classification', 'Training started')
    if project_name:
        operation = 'training_classification'
    else:
        operation = 'training_text_detection'

    api.apipost(operation, customer_name, customer_type, project_name, excel_path, customer_universe)




from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from foqus.cloud_configuration import *

import datetime
import uuid
import calendar
import codecs
import smtplib


# Sending emails
def send_payment_email(customer_email):
    first_name = db.get_profile(DATABASE_USERS_MANAGEMENT_TABLE_NAME, customer_email)[9]
    logger.info("start sending email for informing_client_about_problem_shopify_payment  ...")
    server = smtplib.SMTP(SMTP_HOST, 587)
    server.login(SMTP_USERNAME, SMTP_PASSWORD)
    fromaddr = "FOQUS <hello@foqus.ai>"
    toaddr = customer_email
    msg = MIMEMultipart()
    msg['From'] = fromaddr
    msg['To'] = toaddr
    msg['Subject'] = 'Mettre à jour votre payement '
    html_file = codecs.open(PATH_EMAIL_PROBLEM_SHOPIFY_PAYMENT, 'r')
    payement_url = DOMAIN + 'payement/'
    html = ((str(html_file.read()).replace("#first_name", first_name)).replace("#payement_url", payement_url))
    part2 = MIMEText(html, 'html')
    msg.attach(part2)
    text = msg.as_string()
    server.sendmail(fromaddr, toaddr, text)
    logger.info("End of sending payment email to client %s" % customer_email)


def send_inscription_email(customer_email, full_url, password=None):
    try:
        server = smtplib.SMTP(SMTP_HOST, 587)
        server.starttls()
        server.login(SMTP_USERNAME, SMTP_PASSWORD)
        fromaddr = "FOQUS <contact@trynfit.com>"
        toaddr = customer_email
        msg = MIMEMultipart()
        msg['From'] = fromaddr
        msg['To'] = toaddr
        msg['Subject'] = 'Activez votre compte FOQUS'
        html_file = codecs.open(PATH_INSCRIPTION_EMAIL_CUSTOMER, 'r')
        if password:
            html = (str(html_file.read()).replace("#full_url", full_url).replace('#password', password))
        else:
            html = (str(html_file.read()).replace("#full_url", full_url))

        part2 = MIMEText(html, 'html')
        msg.attach(part2)
        text = msg.as_string()
        server.sendmail(fromaddr, toaddr, text)
        server.quit()
        logger.info("End of sending inscrition email for client %s" % customer_email)
    except Exception as e:
        logger.info("exception in sending email send_inscription_email ..." + str(e))


def send_email_when_training_started(customer_name, project_name, api, subject):

    customer_profile = db.get_customer_info_from_customer_name(customer_name=customer_name)
    for custmer in customer_profile:
        customer_email = custmer[3]

        try:
            logger.info("start sending email of starting training ...")
            server = smtplib.SMTP(SMTP_HOST, 587)
            server.login(SMTP_USERNAME, SMTP_PASSWORD)
            fromaddr = "FOQUS <hello@foqus.ai>"
            toaddr = customer_email
            msg = MIMEMultipart()
            msg['From'] = fromaddr
            msg['To'] = toaddr
            msg['Subject'] = subject
            html_file = codecs.open(PATH_EMAIL_TRAINING_STARTED_CUSTOMER, 'r')
            html = ((str(html_file.read()).replace("#name", (customer_profile[0][9]))).replace("#project_name",
                                                                                               project_name)).replace(
                "#api", api)
            part2 = MIMEText(html, 'html')
            msg.attach(part2)
            text = msg.as_string()
            server.sendmail(fromaddr, toaddr, text)
            logger.info("End of sending email ")
        except Exception as e:
            logger.info("exception in sending email send_email_when_training_started..." + str(e))


def send_email_to_admin(subject, message):
    '''
       Sending to the admin an email with the error of the server
       :param subject: the subject of the email (Error server)
       :param message: the description of the error and where
       :return:
       '''
    try:
        if not LOCAL:
            logger.info("start sending email ...")
            server = smtplib.SMTP(SMTP_HOST, 587)
            server.login(SMTP_USERNAME, SMTP_PASSWORD)
            fromaddr = "FOQUS <hello@foqus.ai>"
            toaddr = ADMIN_EMAIL
            msg = MIMEMultipart()
            msg['From'] = fromaddr
            msg['To'] = toaddr
            msg['Subject'] = subject
            body = message
            msg.attach(MIMEText(body, 'plain'))
            text = msg.as_string()
            server.sendmail(fromaddr, toaddr, text)
            logger.info("End of sending email to admin ")
    except Exception as e:
        logger.info("exception in sending email to admin send_email_to_admin ..." + str(e))


def send_email(customer_name, project_name, subject):

    customer_profile = db.get_customer_info_from_customer_name(customer_name=customer_name)
    for custmer in customer_profile:
        customer_email = custmer[3]

        try:
            logger.info("start sending email ...")
            server = smtplib.SMTP(SMTP_HOST, 587)
            server.login(SMTP_USERNAME, SMTP_PASSWORD)
            fromaddr = "FOQUS <hello@foqus.ai>"
            toaddr = customer_email
            msg = MIMEMultipart()
            msg['From'] = fromaddr
            msg['To'] = toaddr
            msg['Subject'] = subject
            html_file = codecs.open(PATH_EMAIL_CUSTOMER, 'r')
            html = (str(html_file.read()).replace("#name", (customer_profile[0][9]))).replace("#project_name",
                                                                                              project_name)
            part2 = MIMEText(html, 'html')
            msg.attach(part2)
            text = msg.as_string()
            server.sendmail(fromaddr, toaddr, text)
            logger.info("End of sending email ")
        except Exception as e:
            logger.info("exception in sending email send_email..." + str(e))


def send_email_of_client_inscription(customer_email, date_inscription, subject):
    '''
    Sending to the admin an email with the subscribed user
    :param customer_email: the new user email
    :param date_inscription: the date of the user inscription
    :param subject: the subject of the email (New sign up)
    :return:
    '''
    try:
        logger.info("start sending email of client inscription to admin ...")
        server = smtplib.SMTP(SMTP_HOST, 587)
        server.login(SMTP_USERNAME, SMTP_PASSWORD)
        fromaddr = "FOQUS <hello@foqus.ai>"
        toaddr = ADMIN_EMAIL
        msg = MIMEMultipart()
        msg['From'] = fromaddr
        msg['To'] = toaddr
        msg['Subject'] = subject
        html_file = codecs.open(PATH_EMAIL_CLIENT_INSCRIPTION, 'r')
        html = ((str(html_file.read()).replace("#email", customer_email))).replace("#date", date_inscription)
        part2 = MIMEText(html, 'html')
        msg.attach(part2)
        text = msg.as_string()
        server.sendmail(fromaddr, toaddr, text)
        logger.info("End of sending email of client inscription to admin ")
    except Exception as e:
        logger.info("exception in sending email of client inscription ..." + str(e))


def send_email_for_reset_password(customer_email, full_url, first_name_of_customer, mail_subject):

    server = smtplib.SMTP(SMTP_HOST, 587)
    server.login(SMTP_USERNAME, SMTP_PASSWORD)
    try:
        fromaddr = "FOQUS <hello@foqus.ai>"
        msg = MIMEMultipart()
        msg['From'] =fromaddr
        msg['To'] = customer_email
        msg['Subject'] = mail_subject
        html_file = codecs.open(PATH_EMAIL_RESET_PASSWORD,'r')
        html = ((str(html_file.read()).replace("#full_url", full_url)).replace("#first_name", first_name_of_customer))
        part2 = MIMEText(html, 'html')
        msg.attach(part2)
        server.sendmail(fromaddr, customer_email, msg.as_string())
        return True
    except Exception as e:

        logger.error("exception send_email_for_reset_password ......" + str(e))
        return False


def send_email_for_activate_account(first_name, customer_email, full_url):
    '''
    :param first_name: the name of the client
    :param customer_email: the email of the user that just inscripted
    :param full_url: url of activation
    :return:
    '''
    try:
        logger.info("start sending email for account activation ...")
        server = smtplib.SMTP(SMTP_HOST, 587)
        server.login(SMTP_USERNAME, SMTP_PASSWORD)
        fromaddr = "FOQUS <hello@foqus.ai>"
        toaddr = customer_email
        msg = MIMEMultipart()
        msg['From'] = fromaddr
        msg['To'] = toaddr
        msg['Subject'] = 'Activate your account FOQUS'
        html_file = codecs.open(PATH_EMAIL_ACTIVATE_ACCOUNT, 'r')
        html = ((str(html_file.read()).replace("#full_url", full_url)).replace("#first_name", first_name))
        part2 = MIMEText(html, 'html')
        msg.attach(part2)
        text = msg.as_string()
        server.sendmail(fromaddr, toaddr, text)
        logger.info("End of sending inscription email to client %s" % customer_email)
    except Exception as e:
        logger.info("exception in send_email_for_activate_account ..." + str(e))


def send_email_to_client(customer_name, subject, message):
    '''
    :param customer_name: the customer name that will receive the email
    :param subject: the subject of the email
    :param message: depends on the erros that encountred when processing the shopify traning cron either no products in
    store or the number of products passes the limit of his plan
    :return: No return
    '''
    customer_profile = db.get_customer_info_from_customer_name(customer_name=customer_name)
    for custmer in customer_profile:
        customer_email = custmer[3]
        try:
            logger.info("start sending email of client inscription to admin ...")
            server = smtplib.SMTP(SMTP_HOST, 587)
            server.login(SMTP_USERNAME, SMTP_PASSWORD)
            fromaddr = "FOQUS <hello@foqus.ai>"
            toaddr = customer_email
            msg = MIMEMultipart()
            msg['From'] = fromaddr
            msg['To'] = toaddr
            msg['Subject'] = subject
            html_file = codecs.open(PATH_EMAIL_TO_CLIENT, 'r')
            html = str(html_file.read()).replace("#message", message)
            part2 = MIMEText(html, 'html')
            msg.attach(part2)
            text = msg.as_string()
            server.sendmail(fromaddr, toaddr, text)
            logger.info("End of sending email to client  cron shopify")
        except Exception as e:
            logger.info("exception in sending email to client cron shopify ..." + str(e))


def create_or_update_user_apikey(user='anonymous', period_in_hours=72):
    db.update_customer(user, str(uuid.uuid4()), period_in_hours)
    db.commit_db_changes()


def update_customer_apikey(email):
    apikey = db.update_customer_apikey(email=email, apikey=str(uuid.uuid4()))
    return apikey


def specified_project_status(customer_name, customer_type, project_name):
    result = db.get_status_project(table_name=STATUS_PROJECT_TABLE, customer=customer_name,
                                   customer_type=customer_type, api="similars", project=project_name)
    return result


def create_update_project(customer, customer_type, project_to_delete, api, status,name, training_details, counter):
    table_name = STATUS_PROJECT_TABLE
    db.create_status_projects_table(table_name)
    db.insert_or_update_status_projects_table(table_name, customer, customer_type, project_to_delete, api, status,
                                              name, training_details, counter)


def is_apikey_valid(user_apikey):
    try:
        user = db.get_customer_from_apikey(apikey=user_apikey)[0]
    except:
        logger.error("APIKEY not found. Access forbidden for '" + str(user_apikey) + "'")
        return False
    return True


def is_user_allowed(user_apikey):
    try:
        user = db.get_customer_from_apikey(apikey=user_apikey)[0]
    except Exception as e:
        logger.error("exception ...................%s" % e)
        logger.error("APIKEY not found. Access forbidden for '" + str(user_apikey) + "'")
        return False

    expiration = db.get_expiration_from_apikey(apikey=user_apikey)[0]
    if time.time() > expiration:
        logger.info("User '" + user + "' is not allowed because APIKEY has expired. Expiration date: " + time.ctime(
            int(db.get_expiration_from_customer(user)[0])))
        return False
    return True


def get_apikey_expiration(user_apikey):
    try:
        user = db.get_customer_from_apikey(apikey=user_apikey)[0]
    except:
        logger.error("APIKEY not found. Access forbidden for '" + user_apikey + "'")
        return False

    expiration = db.get_expiration_from_apikey(apikey=user_apikey)[0]
    return expiration


def get_user_apikey(customer):
    try:
        user_apikey = db.get_apikey_from_customer(customer=customer)[0]
    except:
        logger.error("Customer not found. Access forbidden for '" + customer + "'")
        return None

    expiration = db.get_expiration_from_apikey(apikey=user_apikey)[0]
    if time.time() > expiration:
        logger.error("User '" + customer + "' is not allowed because APIKEY has expired. Expiration date: " +
                     time.ctime(int(db.get_expiration_from_customer(customer)[0])))
        return None
    return user_apikey


def is_customer_registered(customer):
    try:
        db.get_apikey_from_customer(customer=customer)[0]
    except:
        logger.error("Customer '" + customer + "' not found.")
        return False
    return True


def get_json_ads_number(json_input, customer_name, customer_type):
    return len(json_input[customer_name.lower() + '_' + customer_type.lower()])


def get_json_ad_status(json_input, index):
    return json_input[index]['Status']


def get_json_ad_photos(json_input, customer_name, customer_type, index):
    return json_input[customer_name.lower() + '_' + customer_type.lower()][index]['Photos']


def get_json_ad_categorie(json_input, customer_name, customer_type, index):
    return json_input[customer_name.lower() + '_' + customer_type.lower()][index]['Categorie']


def save_data_from_cms(customer_name, customer_type, project_name, cms, url_shop, access_token):
    db.create_cms_table()
    try:
        db.add_or_update_cms_table(table_name=CMS_TABLE, customer_name=customer_name,
                                   customer_type=customer_type, project_name=project_name, cms=cms, url_shop=url_shop,
                                   token=access_token)
    except Exception as e:
        logger.error("Error in saving the data from cms %s for client %s access_token %s, error %s"
                     % (cms, customer_name, access_token, e))


def scores_repartition_per_users(customer, customer_type):
    response_list = db.select_similars_classification_result(HISTORY_SEARCH_TABLE, customer, customer_type)
    nb_person_10_30 = 0
    nb_person_30_50 = 0
    nb_person_50_75 = 0
    nb_person_75_90 = 0
    nb_person_90_100 = 0
    if response_list != []:
        for j in range(0, len(response_list)):
            for i in range(0, len(response_list[j])):
                if i == 7:
                    if "similars" in response_list[j][i]:
                        similars = json.loads(response_list[j][i])
                        if similars['similars'] == []:
                            continue
                        elif int(similars["similars"][0]['score']) < 30:
                            nb_person_10_30 = nb_person_10_30 + response_list[j][9]
                        elif int(similars["similars"][0]['score']) > 30 and int(
                                similars["similars"][0]['score']) < 50:
                            nb_person_30_50 = nb_person_30_50 + response_list[j][9]
                        elif int(similars["similars"][0]['score']) > 50 and int(
                                similars["similars"][0]['score']) < 75:
                            nb_person_50_75 = nb_person_50_75 + response_list[j][9]
                        elif int(similars["similars"][0]['score']) > 75 and int(
                                similars["similars"][0]['score']) < 90:
                            nb_person_75_90 = nb_person_75_90 + response_list[j][9]
                        else:
                            nb_person_90_100 = nb_person_90_100 + response_list[j][9]
    else:
        logger.info("No historic detected")

    nb_person_list = [nb_person_10_30, nb_person_30_50, nb_person_50_75, nb_person_75_90, nb_person_90_100]
    return nb_person_list



def get_counter_of_last_four_months(customer, customer_type):
    today = datetime.date.today()
    first = today.replace(day=1)
    curr_month = str(today)[5:7]
    curr_month_year = str(today)[5:7]+'-'+str(today)[0:4]
    lastMonth = (first - datetime.timedelta(days=1)).strftime("%m")
    lastMonth_year = (first - datetime.timedelta(days=1)).strftime("%m-%Y")
    first_day_of_last_month = (first - datetime.timedelta(days=1)).replace(day=1)
    last_lastMonth = (first_day_of_last_month - datetime.timedelta(days=1)).strftime("%m")
    last_lastMonth_year = (first_day_of_last_month - datetime.timedelta(days=1)).strftime("%m-%Y")
    first_day_four_month_ago = (
        first_day_of_last_month - datetime.timedelta(days=1) - datetime.timedelta(days=1)).replace(day=1)
    fourth_month_ago = (first_day_four_month_ago - datetime.timedelta(days=1)).strftime("%m")
    fourth_month_ago_year = (first_day_four_month_ago - datetime.timedelta(days=1)).strftime("%m-%Y")
    last_four_last_months = [curr_month,lastMonth, last_lastMonth, fourth_month_ago]
    lastfour_months_year = [curr_month_year,lastMonth_year, last_lastMonth_year, fourth_month_ago_year]

    request_number = []

    for month in lastfour_months_year:
        value = db.select_utilistation_par_mois(HISTORY_SEARCH_TABLE, customer, customer_type, month)

        if value:
            request_number.append(value)
        else:
            request_number.append(0)
    nbr_requests_by_month = []
    for j in range(0, len(request_number)):
        nbr_requests_by_month.append((last_four_last_months[j], request_number[j]))


    #====================utilisation_per_periode

    last_date_in_last_month = first - datetime.timedelta(days=1)
    if str(calendar.monthrange(int(today.strftime("%Y")), int(today.strftime("%m")))[1]) == today.strftime("%d"):
        last_month_day = calendar.monthrange(int(last_date_in_last_month.strftime("%Y")), int(last_date_in_last_month.strftime("%m")))[1]
        try:
            if int(today.strftime("%m")) - 1 == 2:
                current_day_in_the_last_month = today.replace(day=28, month=int(last_date_in_last_month.strftime("%m")))
            else:
                current_day_in_the_last_month = today.replace(day=last_month_day).replace(
                    month=int(last_date_in_last_month.strftime("%m")))

        except Exception as e:
            current_day_in_the_last_month = today.replace(month=int(last_date_in_last_month.strftime("%m"))).replace(day=last_month_day)
    else:
        if int(today.strftime("%m")) - 1 == 2:
            current_day_in_the_last_month = today.replace(day=28, month=int(last_date_in_last_month.strftime("%m")))
        else:
            current_day_in_the_last_month = today.replace(month=int(last_date_in_last_month.strftime("%m")))

    utilisation_per_period = db.get_similars_request_period(HISTORY_SEARCH_TABLE, customer, customer_type, str(first),
                                                            str(today))
    utilisation_per_last_period = db.get_similars_request_period(HISTORY_SEARCH_TABLE, customer, customer_type,
                                                                 str(first_day_of_last_month),
                                                                 str(current_day_in_the_last_month))
    if not utilisation_per_period:
        user_progress_requests = 0
    elif utilisation_per_last_period:
        user_progress_requests = ((utilisation_per_period - utilisation_per_last_period) / utilisation_per_period) * 100
    else:
        user_progress_requests = 100

    user_progress_requests_3_month = db.select_utilistation_par_mois(HISTORY_SEARCH_TABLE, customer, customer_type,
                                                                     last_lastMonth_year)
    user_progress_requests_4_month = db.select_utilistation_par_mois(HISTORY_SEARCH_TABLE, customer, customer_type,
                                                                     fourth_month_ago_year)

    if not user_progress_requests_3_month:
        progress_month_3_ago = 0
    else:
        if user_progress_requests_4_month is not None:
            progress_month_3_ago = ((user_progress_requests_3_month - user_progress_requests_4_month) / user_progress_requests_3_month) * 100
        else:
            progress_month_3_ago = (user_progress_requests_3_month / user_progress_requests_3_month) * 100

    return nbr_requests_by_month, user_progress_requests, progress_month_3_ago


def user_exist(user_apikey, customer, customer_type):
    result = db.get_customer_name_and_type(apikey=user_apikey, customer=customer, customer_type=customer_type)
    if result:
        return True
    else:
        return False


def project_exist(customer_name, customer_type, api, project):
    result = db.get_project_and_status(customer=customer_name, customer_type=customer_type, api=api, project=project)
    return result


def number_request_by_week(customer, customer_type):
    today = datetime.date.today()

    current_year = str(today)[0:4]
    num_post_week = db.get_date_counter(HISTORY_SEARCH_TABLE, customer, customer_type, current_year)
    week_list_and_value = {}
    keys=[]
    values=[]
    for t in num_post_week:
        num_week =(t[1].isocalendar()[1])
        if (num_week) in week_list_and_value.keys():
            week_list_and_value[num_week] += t[0]
        else:
            week_list_and_value[num_week] = t[0]

    for key, value in sorted(week_list_and_value.items()):
        keys.append(key)
        values.append(value)

    nbr_request_by_week=[]
    for j in range(0, len(keys)):
        nbr_request_by_week.append({'week': str(keys[j]), 'number': values[j]})

    return nbr_request_by_week


def get_photos(json_input, customer_name, customer_type):
    photos = []
    try:
        for i in range(get_json_ads_number(json_input, customer_name, customer_type)):
            photos = photos + get_json_ad_photos(json_input, customer_name, customer_type, i)
    except Exception as e:
        logger.error(e)
    return photos


""" This module contains an implementation of a Vantage Point-tree (VP-tree)."""
import numpy as np

class VPTree:
    """ VP-Tree data structure for efficient nearest neighbor search.

    The VP-tree is a data structure for efficient nearest neighbor
    searching and finds the nearest neighbor in O(log n)
    complexity given a tree constructed of n data points. Construction
    complexity is O(n log n).

    Parameters
    ----------
    points : Iterable
        Construction points.
    dist_fn : Callable
        Function taking to point instances as arguments and returning
        the distance between them.
    leaf_size : int
        Minimum number of points in leaves (IGNORED).
    """

    def __init__(self, points, dist_fn):
        self.left = None
        self.right = None
        self.left_min = np.inf
        self.left_max = 0
        self.right_min = np.inf
        self.right_max = 0
        self.dist_fn = dist_fn

        if not len(points):
            raise ValueError('Points can not be empty.')

        # Vantage point is point furthest from parent vp.
        vp_i = 0
        self.vp = points[vp_i]
        points = np.delete(points, vp_i, axis=0)

        if len(points) == 0:
            return

        # Choose division boundary at median of distances.
        distances = [self.dist_fn([float(pi) for pi in self.vp], [float(pi) for pi in p]) for p in points]

        median = np.median(distances)

        left_points = []
        right_points = []
        for point, distance in zip(points, distances):
            if distance >= median:
                self.right_min = min(distance, self.right_min)
                if distance > self.right_max:
                    self.right_max = distance
                    right_points.insert(0, point)  # put furthest first
                else:
                    right_points.append(point)
            else:
                self.left_min = min(distance, self.left_min)
                if distance > self.left_max:
                    self.left_max = distance
                    left_points.insert(0, point)  # put furthest first
                else:
                    left_points.append(point)

        if len(left_points) > 0:
            self.left = VPTree(points=left_points, dist_fn=self.dist_fn)

        if len(right_points) > 0:
            self.right = VPTree(points=right_points, dist_fn=self.dist_fn)

    def _is_leaf(self):
        return (self.left is None) and (self.right is None)

    def get_nearest_neighbor(self, query):
        """ Get single nearest neighbor.

        Parameters
        ----------
        query : Any
            Query point.

        Returns
        -------
        Any
            Single nearest neighbor.
        """
        return self.get_n_nearest_neighbors(query, n_neighbors=1)[0]

    def get_n_nearest_neighbors(self, query, n_neighbors):
        """ Get `n_neighbors` nearest neigbors to `query`

        Parameters
        ----------
        query : Any
            Query point.
        n_neighbors : int
            Number of neighbors to fetch.

        Returns
        -------
        list
            List of `n_neighbors` nearest neighbors.
        """
        if not isinstance(n_neighbors, int) or n_neighbors < 1:
            raise ValueError('n_neighbors must be strictly positive integer')
        neighbors = _AutoSortingList(max_size=n_neighbors)
        nodes_to_visit = [(self, 0)]

        furthest_d = np.inf

        while len(nodes_to_visit) > 0:
            node, d0 = nodes_to_visit.pop(0)
            if node is None or d0 > furthest_d:
                continue

            d = self.dist_fn(query, node.vp)
            if d < furthest_d:
                neighbors.append((d, node.vp))
                furthest_d, _ = neighbors[-1]

            if node._is_leaf():
                continue

            if node.left_min <= d <= node.left_max:
                nodes_to_visit.insert(0, (node.left, 0))
            elif node.left_min - furthest_d <= d <= node.left_max + furthest_d:
                nodes_to_visit.append((node.left,
                                       node.left_min - d if d < node.left_min
                                       else d - node.left_max))

            if node.right_min <= d <= node.right_max:
                nodes_to_visit.insert(0, (node.right, 0))
            elif node.right_min - furthest_d <= d <= node.right_max + furthest_d:
                nodes_to_visit.append((node.right,
                                       node.right_min - d if d < node.right_min
                                       else d - node.right_max))

        return list(neighbors)

    def get_all_in_range(self, query, max_distance):
        """ Find all neighbours within `max_distance`.

        Parameters
        ----------
        query : Any
            Query point.
        max_distance : float
            Threshold distance for query.

        Returns
        -------
        neighbors : list
            List of points within `max_distance`.

        Notes
        -----
        Returned neighbors are not sorted according to distance.
        """
        neighbors = list()
        nodes_to_visit = [(self, 0)]

        while len(nodes_to_visit) > 0:
            node, d0 = nodes_to_visit.pop(0)
            if node is None or d0 > max_distance:
                continue

            d = self.dist_fn([float(pi) for pi in query], [float(pi) for pi in node.vp])
            if d < max_distance:
                neighbors.append((d, node.vp))

            if node._is_leaf():
                continue

            if node.left_min <= d <= node.left_max:
                nodes_to_visit.insert(0, (node.left, 0))
            elif node.left_min - max_distance <= d <= node.left_max + max_distance:
                nodes_to_visit.append((node.left,
                                       node.left_min - d if d < node.left_min
                                       else d - node.left_max))

            if node.right_min <= d <= node.right_max:
                nodes_to_visit.insert(0, (node.right, 0))
            elif node.right_min - max_distance <= d <= node.right_max + max_distance:
                nodes_to_visit.append((node.right,
                                       node.right_min - d if d < node.right_min
                                       else d - node.right_max))

        return neighbors


class _AutoSortingList(list):
    """ Simple auto-sorting list.

    Inefficient for large sizes since the queue is sorted at
    each push.

    Parameters
    ---------
    size : int, optional
        Max queue size.
    """

    def __init__(self, max_size=None, *args):
        super(_AutoSortingList, self).__init__(*args)
        self.max_size = max_size

    def append(self, item):
        """ Append `item` and sort.

        Parameters
        ----------
        item : Any
            Input item.
        """
        super(_AutoSortingList, self).append(item)
        self.sort(key=lambda x: x[0])
        if self.max_size is not None and len(self) > self.max_size:
            self.pop()

